% Driver program for Gibbs sampler estimator auxiliary variables model
%
% Keith Sill
% November 2007
%
%

%path(path,'/home/keith/MATLAB');

clear;

% structure to hold para names, startvals, estim vals, and stds
paras = struct('name',...
    {'alp','zeta_p','iota_p','del','ups','Bigphi','s2','h','a2','nu_l',...
    'nu_m','zeta_w','iota_w','law','beta','psi1','psi2','rho_r','pistar',...
    'gam','Wadj','chi','laf','gstar','Ladj','rho_z','rho_phi','rho_chi',...
    'rho_laf','rho_mu','rho_b','rho_g','sig_z','sig_phi','sig_chi',...
    'sig_laf','sig_mu','sig_b','sig_g','sig_r'},...
    'sval',...
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},...
    'estval',...
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},...
    'stderr',...
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0});

% get the prior from csv file

[prior] = csvread('benchmark_prior.csv',16,1);
pshape   = prior(:,1);
pmean    = prior(:,2);
pstdd    = prior(:,3);
pmask    = prior(:,4);
pfix     = prior(:,5);
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

% get the maximization start vals from csv file and put them in the paras
% structure

[p0vals] = csvread('benchmark_start.csv',2,1);
for i = 1:length(p0vals)
    paras(i).sval = p0vals(i);
end

%% Load data parameter draws, calculate mean

nestim = 27;
nstep = 12;
fcastmat = zeros(nestim+1,nstep+1);

mean_mat = zeros(70,40);

nblocks = 70;
nsim    = 1000;
npara   = 40;

paramat = zeros(nblocks*nsim,npara);
trej = 0;

loaddate = 'output-17-Mar-2009/';

tic;
        
trej = 0;

for i = 1:nblocks

    clear fname parasim;

    fname = strcat(loaddate,'smpl-1984Q1+',num2str(94),'--block-',num2str(i+(100-nblocks)));
    load(fname,'parasim','rej');
    ridx1 = (i-1)*nsim + 1;
    ridx2 = i*nsim;
    paramat(ridx1:ridx2,:) = parasim;
    trej = sum(rej) + trej;

end

% mean and standard deviation of draws
mpara = mean(paramat);
spara = std(paramat);
rej_pct = trej/(nblocks*nsim);

% fill the structure with mean of draws and std of draws
for i = 1:npara
    paras(i).estval = mpara(i);
    paras(i).stderr = spara(i);
end

% structure to hold values governing para fixity
varargin = struct('pmaskinv',pmaskinv,'pfix',pfix,'pmask',pmask,'pmean',...
     pmean,'pstdd',pstdd,'pshape',pshape);

    % display the estimates, standard errors, start values, prior mean, prior
    % std, whether para is fixed, and value fixed at (if fixed)
% disp(sprintf('%15s%15s%15s%15s%15s%15s%15s%15s','Parameter','Estimate','Std','Start-V','Prior Mean',...
%     'Prior Std','mask','fixed'));
% for i = 1:npara
%     disp(sprintf('%15s%15.5g%15.5g%15.5g%15.5g%15.5g%15.5g%15.5g',paras(i).name,paras(i).estval,paras(i).stderr,...
%          paras(i).sval,varargin.pmean(i),varargin.pstdd(i),varargin.pmask(i),varargin.pfix(i)));
% end
% 
% disp(sprintf('%15s%15g','Rejection Pct',rej_pct));

% Form the reduced-state selection matrix
%
% The state variable ordering is:
% y,c,i,k,,bar,L,m,mc,pi,R,rk,u,w,wtil,xi,xik,z,phi,mu,b,g,laf,Ec,Epi,
% Erk,Ew,Ewtil,Exi,Exik,Ei,yl,cl,il,wl,ml
%
% The 'true' state variables are c,i,kbar,R,w,z,phi,mu,b,g,laf which are
% indexed as 2,3,5,10,13,17,18,19,20,21,22
%
% note that pi is not a state when the estimated model shuts down inflation
% indexation, as we have done here.
%
M = zeros(35,11);
M(2,1)   = 1;
M(3,2)   = 1;
M(5,3)   = 1;
M(10,4)  = 1;
M(13,5)  = 1;
M(17,6)  = 1;
M(18,7)  = 1;
M(19,8)  = 1;
M(20,9)  = 1;
M(21,10) = 1;
M(22,11) = 1;

% load estimation sample data 1984Q1 to 2007Q3
str1 = 186+27;
[YY,txt,raw] = xlsread('usModelData.xls','Sheet3',strcat('a119:h',num2str(str1)));


[smat,xvar,ypred,evec,lambda,retcode,Pt0] = genrstates(mpara,YY,M);
[nobs,nstates] = size(xvar);

% Load the auxiliary variables
% Estimation sample data 1984Q1 to 200Q4 
str2 = 168+27;
[data1,txt1,raw1] = xlsread('auxvars2.xls','Sheet2',strcat('a101:e',num2str(str2)));


% Pre-sample data
%data0 = csvread('auxvars.csv',2,1,[2,1,100,3]);

% Sample data
hs  = data1(:,4)/1000;

%% Housing Starts model
yvar = hs;
a0   = 1.532;  % mean value of starts 1959Q2-1983Q4
       
% 
% Prior from regression of Investment state on fundamental states

%mu1 = inv(xvar'*xvar)*xvar'*smat(:,3);
iota    = zeros(35,1);
iota(3) = 1;
mu01    = inv(M'*Pt0*M);
mu02    = M' * Pt0 * iota;
mu0     = mu01 * mu02 ;

        
scale = sqrt(var(yvar)/var(smat(:,3)));

% constant as suggested from measurement equation and parameter estimates
mu0 = scale*[a0/scale; mu0];

% Prior variance from AR(1) regression
yar = yvar(2:end);
[nobs,nvar]=size(yar);
xar = yvar(1:end-1);
bar = inv(xar'*xar)*xar'*yar;
ear = yar - xar*bar;
sig0 = ear'*ear/nobs;

V00 = inv(diag([1;lambda']));

% iterate on lambda ? set # here
nlam = [1, 0.1, 0.00001];

sa1 = [];
omat = [];

for ilam = 1:max(size(nlam))
  
    % Set hyperparameters
    lam1 = nlam(ilam);
    lam2 = nlam(ilam);
    lam3 = nlam(ilam);
    hypermat = diag([lam1,lam2,lam3,lam2,lam2,lam2,lam2,lam2,lam2,lam2,lam2,lam2]);

% 
    v0   = hypermat * V00;

    % 
    %% Run the Gibbs sampler
    niter = 10000;
    burnin = 1000;
    tau = sqrt(0.15*var(yvar));
    %disp(sprintf('%10s%10f','tau is: ',tau));
    [pvec_hs,hb,predicted_hs,bhat] = gibbs_sampler(yvar,xvar,mu0,v0,sig0,niter,burnin,tau);
    
    % marginal likelihood for lambda
    ahat    = pvec_hs(1:12);
    rhohat  = pvec_hs(13);
    sighat2 = pvec_hs(14);
    s = tau;
    v = 2;
    
    [mlh2,a1,a2,a3,a4,b1,b2,b3] = margLambdaV2(yvar,xvar,ahat,rhohat,sighat2,mu0,v0,v,s,bhat,niter,burnin,tau);
    %[mlh1,c1,c2,c3,c4,d1,d2,d3] = margLambdaV1(yvar,xvar,ahat,rhohat,sighat2,mu0,v0,bhat,niter,burnin,tau);
    
    blk     = -nobs/2*log(2*pi) - nobs/2*log(sighat2) - nobs/2;
    
    disp(sprintf('\nHousing starts model'))
    disp(sprintf('For Lambda = %5.2g the marginal log likelihood is %5.2f',nlam(ilam),mlh2))
    disp(sprintf('The benchmark log likelihood is %5.2f',blk))
    disp(sprintf('The value of tau is %5.4f',tau))

%     omat = [omat;mlh2,a1,a2,a3,a4,b1,b2,b3; ...
%                 mlh1,c1,c2,c3,c4,d1,d2,d3];
%     
    
    sa1(:,ilam) = pvec_hs(1) + smat * M * pvec_hs(2:12)';
    eta(:,ilam) = yvar - sa1(:,ilam);
    sn_hs = var(sa1(:,ilam))/var(eta(:,ilam));
    mlh_hs = mlh2;
    hb_hs  = hb;
    
%     % insert code to save pvec_pce
     fname = strcat(date,'pvec_hs-lambda-',num2str(ilam));
     save(fname,'pvec_hs','hb_hs','sn_hs','mlh_hs');
    
    
      
     pvec = pvec_hs;
     
     % The 'true' state variables are c,i,kbar,R,w,z,phi,mu,b,g,laf
    
%     disp(sprintf('\nHousing Starts Model Estimates for lambda = %g\n',nlam(ilam)))
%     disp(sprintf(' coefficient on: & estimate  & 90 pct prob interval'))
%     disp(sprintf(' a0              &  %6.3f    & [%5.3f,%5.3f]',pvec(1),hb(1,1),hb(2,1)))
%     disp(sprintf(' c               &  %6.3f    & [%5.3f,%5.3f]',pvec(2),hb(1,2),hb(2,2)))
%     disp(sprintf(' i               &  %6.3f    & [%5.3f,%5.3f]',pvec(3),hb(1,3),hb(2,3)))
%     disp(sprintf(' kbar            &  %6.3f    & [%5.3f,%5.3f]',pvec(4),hb(1,4),hb(2,4)))
%     disp(sprintf(' R               &  %6.3f    & [%5.3f,%5.3f]',pvec(5),hb(1,5),hb(2,5)))
%     disp(sprintf(' w               &  %6.3f    & [%5.3f,%5.3f]',pvec(6),hb(1,6),hb(2,6)))
%     disp(sprintf(' z               &  %6.3f    & [%5.3f,%5.3f]',pvec(7),hb(1,7),hb(2,7)))
%     disp(sprintf(' phi             &  %6.3f    & [%5.3f,%5.3f]',pvec(8),hb(1,8),hb(2,8)))
%     disp(sprintf(' mu              &  %6.3f    & [%5.3f,%5.3f]',pvec(9),hb(1,9),hb(2,9)))
%     disp(sprintf(' b               &  %6.3f    & [%5.3f,%5.3f]',pvec(10),hb(1,10),hb(2,10)))
%     disp(sprintf(' g               &  %6.3f    & [%5.3f,%5.3f]',pvec(11),hb(1,11),hb(2,11)))
%     disp(sprintf(' laf             &  %6.3f    & [%5.3f,%5.3f]',pvec(12),hb(1,12),hb(2,12)))
%     disp(sprintf(' rho             &  %6.3f    & [%5.3f,%5.3f]',pvec(13),hb(1,13),hb(2,13)))
%     disp(sprintf(' sig             &  %6.3f    & [%5.3f,%5.3f]',pvec(14),hb(1,14),hb(2,14)))
% 
% 
% 
% 
% 
% disp(sprintf('\nHousing Starts Model mu(alpha,0) Estimates'))
%     disp(sprintf(' coefficient on: & estimate'))
%     disp(sprintf(' a0              &  %6.3f    ',mu0(1)))
%     disp(sprintf(' c               &  %6.3f    ',mu0(2)))
%     disp(sprintf(' i               &  %6.3f    ',mu0(3)))
%     disp(sprintf(' kbar            &  %6.3f    ',mu0(4)))
%     disp(sprintf(' R               &  %6.3f    ',mu0(5)))
%     disp(sprintf(' w               &  %6.3f    ',mu0(6)))
%     disp(sprintf(' z               &  %6.3f    ',mu0(7)))
%     disp(sprintf(' phi             &  %6.3f    ',mu0(8)))
%     disp(sprintf(' mu              &  %6.3f    ',mu0(9)))
%     disp(sprintf(' b               &  %6.3f    ',mu0(10)))
%     disp(sprintf(' g               &  %6.3f    ',mu0(11)))
%     disp(sprintf(' laf             &  %6.3f    ',mu0(12)))
    
end
%%

% disp(sprintf('Marginal likelihoods for Housing Starts'))
% disp(sprintf('Proper Prior? lambda  mlh   n1       n2      n3     n4      d1     d2    d3'))
% disp(sprintf('______________________________________________________________________________'))
% disp(sprintf('yes (v=2)\t|\t%5.3g\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f',1.0,omat(1,:)))
% disp(sprintf('no\t\t\t|\t%5.3g\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f',1.0,omat(2,:)))
% disp(sprintf('______________________________________________________________________________'))
% disp(sprintf('yes (v=2)\t|\t%5.3g\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f',0.1,omat(3,:)))
% disp(sprintf('no\t\t\t|\t%5.3g\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f',0.1,omat(4,:)))
% disp(sprintf('______________________________________________________________________________'))
% disp(sprintf('yes (v=2)\t|\t%5.3g\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f',1e-5,omat(5,:)))
% disp(sprintf('no\t\t\t|\t%5.3g\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f\t%5.3f',1e-5,omat(6,:)))



ti = 1984.0:0.25:2007.5;
edx = find(1984.0==ti);
mhs = hs;
% 
% figure('Name','Housing Starts 1');
% lmb = 1;
% plot(ti(edx:end),mhs(edx:end,1),ti(edx:end),sa1(edx:end,lmb));
% title(strcat('a1*s(t|t) vs, de-meaned housing starts for lambda = ',num2str(nlam(lmb))));
% axis([1980,2010,-0.6,1.6]);
% legend('actual','predicted');

%%      


figure('Name','Housing Starts 2');
plot(ti(edx:end),mhs(edx:end),'b-',...
    ti(edx:end),sa1(edx:end,1),'k-.',...
    ti(edx:end),sa1(edx:end,3),'r--');
legend('actual','{\lambda}=1','{\lambda}=1e-5','Location','SouthWest');
legend boxoff;
title('Housing Starts');


