% Driver program for Gibbs sampler estimator auxiliary variables model
%
% Keith Sill
% November 2007
%
%

%path(path,'/home/keith/MATLAB');

clear;

% structure to hold para names, startvals, estim vals, and stds
paras = struct('name',...
    {'alp','zeta_p','iota_p','del','ups','Bigphi','s2','h','a2','nu_l',...
    'nu_m','zeta_w','iota_w','law','beta','psi1','psi2','rho_r','pistar',...
    'gam','Wadj','chi','laf','gstar','Ladj','rho_z','rho_phi','rho_chi',...
    'rho_laf','rho_mu','rho_b','rho_g','sig_z','sig_phi','sig_chi',...
    'sig_laf','sig_mu','sig_b','sig_g','sig_r'},...
    'sval',...
    {0},...
    'estval',...
    {0},...
    'stderr',...
    {0});

% get the prior from csv file

[prior] = csvread('benchmark_prior.csv',16,1);
pshape   = prior(:,1);
pmean    = prior(:,2);
pstdd    = prior(:,3);
pmask    = prior(:,4);
pfix     = prior(:,5);
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

% get the maximization start vals from csv file and put them in the paras
% structure

[p0vals] = csvread('benchmark_start.csv',2,1);
for i = 1:length(p0vals)
    paras(i).sval = p0vals(i);
end

%% Load data parameter draws, calculate mean
date = '17-Mar-2009';
datename = strcat('output-',date,'/');
mkdir(datename);

nestim = 27;
nstep = 12;
fcastmat = zeros(3,nestim+1,nstep);

mean_mat = zeros(70,40);

nblocks = 70;
nsim    = 1000;
npara   = 40;

niter   = 10000;
nskip   = 50;
burnin  = 1000;
ndraws  = floor((niter-burnin)/nskip);
paraDSGE = zeros(ndraws,40);


paramat = zeros(nblocks*nsim,npara);
trej = 0;

loaddate = 'output-17-Mar-2009/';

tic;

% iterate on lambda ? set # here
nlam = [1, 0.1, 0.00001];

% actuals for computing rmse's at 1, 2, 4, and 12 steps. Make sure the
% right column is selected at the bottom of the code for calculating rmse

sa1  = xlsread('auxvars2.xls','Sheet2','a169:e196');
sa2  = xlsread('auxvars2.xls','Sheet2','a170:e196');
sa4  = xlsread('auxvars2.xls','Sheet2','a172:e196');
sa12 = xlsread('auxvars2.xls','Sheet2','a180:e196');

for ilam = 1:max(size(nlam))
    
  
    
    for j = 0:nestim
        
        trej = 0;

        for i = 1:nblocks

            clear fname parasim;

            fname = strcat(loaddate,'smpl-1984Q1+',num2str(67+j),'--block-',num2str(i+(100-nblocks)));
            load(fname,'parasim','rej');
            ridx1 = (i-1)*nsim + 1;
            ridx2 = i*nsim;
            paramat(ridx1:ridx2,:) = parasim;
            trej = sum(rej) + trej;

        end

        % mean and standard deviation of draws
        mpara = mean(paramat(1000:end,:));
        spara = std(paramat);
        rej_pct = trej/(nblocks*nsim);

        % fill the structure with mean of draws and std of draws
        for i = 1:npara
            paras(i).estval = mpara(i);
            paras(i).stderr = spara(i);
        end

        % structure to hold values governing para fixity
        varargin = struct('pmaskinv',pmaskinv,'pfix',pfix,'pmask',pmask,'pmean',...
             pmean,'pstdd',pstdd,'pshape',pshape);

        % display the estimates, standard errors, start values, prior mean, prior
        % std, whether para is fixed, and value fixed at (if fixed)
    %     disp(sprintf('%15s%15s%15s%15s%15s%15s%15s%15s','Parameter','Estimate','Std','Start-V','Prior Mean',...
    %         'Prior Std','mask','fixed'));
    %     for i = 1:npara
    %         disp(sprintf('%15s%15.5g%15.5g%15.5g%15.5g%15.5g%15.5g%15.5g',paras(i).name,paras(i).estval,paras(i).stderr,...
    %             paras(i).sval,varargin.pmean(i),varargin.pstdd(i),varargin.pmask(i),varargin.pfix(i)));
    %     end

        disp(sprintf('%15s%15g','Rejection Pct',rej_pct));
        disp(sprintf('%15s%15g','lambda is ',nlam(ilam)));
        disp(sprintf('%15s%10d%10s%10d','Iteration ',j,'of ',nestim));


        % Form the reduced-state selection matrix
        %
        % The state variable ordering is:
        % y,c,i,k,,bar,L,m,mc,pi,R,rk,u,w,wtil,xi,xik,z,phi,mu,b,g,laf,Ec,Epi,
        % Erk,Ew,Ewtil,Exi,Exik,Ei,yl,cl,il,wl,ml
        %
        % The 'true' state variables are c,i,kbar,R,w,z,phi,mu,b,g,laf which are
        % indexed as 2,3,5,10,13,17,18,19,20,21,22
        %
        % note that pi is not a state when the estimated model shuts down inflation
        % indexation, as we have done here.
        %
        M = zeros(35,11);
        M(2,1)   = 1;
        M(3,2)   = 1;
        M(5,3)   = 1;
        M(10,4)  = 1;
        M(13,5)  = 1;
        M(17,6)  = 1;
        M(18,7)  = 1;
        M(19,8)  = 1;
        M(20,9)  = 1;
        M(21,10) = 1;
        M(22,11) = 1;

        % load estimation sample data 1984Q1 to 2000Q4+j
        str1 = 186+j;
        [YY,txt,raw] = xlsread('usModelData.xls','Sheet3',strcat('a119:h',num2str(str1)));

        [smat,xvar,ypred,evec,lambda,retcode] = genrstates(mpara,YY,M);
        [nobs,nstates] = size(xvar);

        % Load the auxiliary variables
        % Estimation sample data 1984Q1 to 2000Q4+j 
       
        str2 = 168+j;
        [data1,txt1,raw1] = xlsread('auxvars2.xls','Sheet2',strcat('a101:e',num2str(str2)));
        
        % Sample data
        yvar  = data1(:,2);

        %% CPCE inflation model

        pistar = exp(paras(19).estval/400);
        % 
        % Prior from regression of pi state on fundamental states

        mu0 = inv(xvar'*xvar)*xvar'*smat(:,9);

        % constant as suggested from measurement equation and parameter estimates
        mu0 = [100*log(pistar); mu0];

        % Prior variance from AR(1) regression
        yar = yvar(2:end);
        [nobs,nvar]=size(yar);
        xar = yvar(1:end-1);
        bar = inv(xar'*xar)*xar'*yar;
        ear = yar - xar*bar;
        sig0 = ear'*ear/nobs;

        V00 = inv(diag([1;lambda']));

        % Set hyperparameters
        lam1 = nlam(ilam);
        lam2 = nlam(ilam);
        lam3 = nlam(ilam);
        hypermat = diag([lam1,lam2,lam3,lam2,lam2,lam2,lam2,lam2,lam2,lam2,lam2,lam2]);

        % 
        v0   = hypermat * V00;

        % 
        %% Run the Gibbs sampler and forecast
        tau = sqrt(0.15*var(yvar));
        disp(sprintf('%10s%10f','tau is: ',tau));
        bhat   = gibbs_lsq(niter,yvar,xvar,mu0,v0,sig0,tau);
        fname = strcat(datename,'bhat-cpce-',num2str(ilam),'-',num2str(j));
        save(fname,'bhat');
        
        %[smat,retcode,DD,ZZ] = state_fcst(mpara,YY,nstep);

        % smat has the same time dimension as YY 
        
        yvar0 = yvar(end,1);
        ymat = zeros(ndraws,nstep);
        
        state = smat(end,:);
        
        for dcount = 1:ndraws
            
            epsDSGE = zeros(nstep,7);
            epsAUX  = randn(nstep,1);
            
            idx   = burnin + nskip * dcount;
            pvec  = bhat(idx,:);
            %ypred = fcast_auxvars2(pvec,yvar0,smat,M,nstep);
            ypred = pdrawsAUX(mpara,pvec,yvar0,state,M,epsDSGE,epsAUX,nstep);
            ymat(dcount,:) = ypred';
            
        end
        
        fcastmat(ilam,j+1,:) = mean(ymat);
         
        % AR1 forecasts
        yar1          = ar1_fcst2(yvar,nstep);
        ar1mat(j+1,:) = yar1;


    end
    
    %% actual data for forecast sample is 2001Q1 to 2007Q4
    
    s1  = fcastmat(ilam,1:end,1)'     -  sa1(:,2);
    s2  = fcastmat(ilam,1:end-1,2)'   -  sa2(:,2);
    s4  = fcastmat(ilam,1:end-3,4)'   -  sa4(:,2);
    s12 = fcastmat(ilam,1:end-11,12)' -  sa12(:,2);

    rmse1  = sqrt( s1'*s1/max(size(s1)) );
    rmse2  = sqrt(s2'*s2/max(size(s2)));
    rmse4  = sqrt(s4'*s4/max(size(s4)));
    rmse12 = sqrt(s12'*s12/max(size(s12)));
    
    mat(1,ilam) = rmse1;
    mat(2,ilam) = rmse2;
    mat(3,ilam) = rmse4;
    mat(4,ilam) = rmse12;

end

s1  = ar1mat(1:end,1)     -  sa1(:,2);
s2  = ar1mat(1:end-1,2)   -  sa2(:,2);
s4  = ar1mat(1:end-3,4)   -  sa4(:,2);
s12 = ar1mat(1:end-11,12) -  sa12(:,2);

rmse1  = sqrt( s1'*s1/max(size(s1)) );
rmse2  = sqrt(s2'*s2/max(size(s2)));
rmse4  = sqrt(s4'*s4/max(size(s4)));
rmse12 = sqrt(s12'*s12/max(size(s12)));

mat(1,ilam+1) = rmse1;
mat(2,ilam+1) = rmse2;
mat(3,ilam+1) = rmse4;
mat(4,ilam+1) = rmse12;

mat = mat';

disp(sprintf('Forecast comparison table'))
disp(sprintf('(lam0,lam1}) h =1    h = 2    h = 4    h = 12'))
disp(sprintf('(1.0,1.0)   %5.2f   %5.2f    %5.2f    %5.2f', mat(1,:)))
disp(sprintf('(0.1,0.1)   %5.2f   %5.2f    %5.2f    %5.2f', mat(2,:)))
disp(sprintf('(1e-5,1e-5) %5.2f   %5.2f    %5.2f    %5.2f', mat(3,:)))
disp(sprintf('AR(1)       %5.2f   %5.2f    %5.2f    %5.2f', mat(4,:)))

%% Diebold-Mariano

dm = zeros(3,4,4);

% Set variable index here:
vi = 2;

dmLossFnIndex = 1;
dmTestType    = 'two-sided' ;

for ilam = 1:3
    
   dmForecast1 = fcastmat(ilam,1:end,1)';
   dmForecast2 = ar1mat(1:end,1,1);
   dmActual    = sa1(:,vi);

   [DM, MDM, DM_pval, MDM_pval] = ...
               Diebold_Mariano(dmForecast1, dmForecast2, dmActual, ...
                               dmLossFnIndex, dmTestType, 1);
   dm(ilam,1,:) = [DM,MDM,DM_pval,MDM_pval];                        
   
                           
   dmForecast1 = fcastmat(ilam,1:end-1,2)';
   dmForecast2 = ar1mat(1:end-1,2);
   dmActual    = sa2(:,vi);

   [DM, MDM, DM_pval, MDM_pval] = ...
               Diebold_Mariano(dmForecast1, dmForecast2, dmActual, ...
                               dmLossFnIndex, dmTestType, 2);
   dm(ilam,2,:) = [DM,MDM,DM_pval,MDM_pval];                        
   
   dmForecast1 = fcastmat(ilam,1:end-3,4)';
   dmForecast2 = ar1mat(1:end-3,4);
   dmActual    = sa4(:,vi);

   [DM, MDM, DM_pval, MDM_pval] = ...
               Diebold_Mariano(dmForecast1, dmForecast2, dmActual, ...
                               dmLossFnIndex, dmTestType, 4);
   dm(ilam,3,:) = [DM,MDM,DM_pval,MDM_pval];                        
   
   dmForecast1 = fcastmat(ilam,1:end-11,12)';
   dmForecast2 = ar1mat(1:end-11,12);
   dmActual    = sa12(:,vi);

   [DM, MDM, DM_pval, MDM_pval] = ...
               Diebold_Mariano(dmForecast1, dmForecast2, dmActual, ...
                               dmLossFnIndex, dmTestType, 12);
                
   dm(ilam,4,:) = [DM,MDM,DM_pval,MDM_pval];                  
                           
end

cpceMat = [dm(1,1,4),dm(1,2,4),dm(1,3,4),dm(1,4,4);
           dm(2,1,4),dm(2,2,4),dm(2,3,4),dm(2,4,4);
           dm(3,1,4),dm(3,2,4),dm(3,3,4),dm(3,4,4)];
   
mat_cpce = mat;
dm_cpce  = cpceMat;
fname = 'table6_cpce';
save(fname,'mat_cpce','dm_cpce');