% Estimate Auxiliary Models
%

clear;

% Model setup file is run first
[M,YY,AY] = FcstSetup();

loaddate = 'output-4-Feb-2009/';

% Get subsample draws from DSGE model estimation
dname   = 'output-4-Feb-2009/smpl-1984Q1-2007Q4-block-';
nblocks = 100;
nsim    = 10000;
nskip   = 100;
ndraws  = (nsim*nblocks)/nskip;
npara   = 40;
pmat    = SubsampleDsgeDraws(dname,nblocks,nsim,nskip,npara);

pmat    = pmat(0.2*ndraws:end,:);
mpara   = mean(pmat);

% Generate state observations from Kalman filter
[smat,xvar,ypred,evec,lambda,retcode] = genrstates(mpara,YY,M);

% iterate on lambda ? set # here and # of simulations
nlam  = [1, 0.01, 0.00001 ];
niter = 100000;

% Estimation sample data (may differ from FcstSetup output). Here we
% estimate using 1984Q1-2007Q4
AY = AY(1:end-3,:);
[nr,nc] = size(AY);
YY = YY(1:nr,:);

disp('EXISTING DATA FILES WILL BE DELETED, ELSE KILL');
disp('hit any key to continue');
pause;

%% PCE inflation model

fname = strcat(loaddate,'auxpce.dat'); delete(fname);
fid   = fopen(fname,'a');

pce   = AY(:,1);

yvar   = pce;
pistar = exp(mpara(19)/400);

% Prior is from regression of pi state on fundamental states
mu0 = inv(xvar'*xvar)*xvar'*smat(:,9);

% constant as suggested from measurement equation and parameter estimates
mu0 = [100*log(pistar); mu0];

% Prior variance from AR(1) regression
yar = yvar(2:end);
[nobs,nvar]=size(yar);
xar = yvar(1:end-1);
bar = inv(xar'*xar)*xar'*yar;
ear = yar - xar*bar;
sig0 = ear'*ear/nobs;

V00 = inv(diag([1;lambda']));

for ilam = 1:3
    
    disp(sprintf('\n'));
    disp(sprintf('%30s\t%2.1e','--pce inflation-- lambda = ',nlam(ilam)));
    
    % Set hyperparameters
    lam1 = nlam(ilam);
    lam2 = nlam(ilam);
    lam3 = nlam(ilam);
    hypermat = diag([lam1,lam2,lam3,lam2,lam2,lam2,lam2,lam2,lam2,lam2,lam2,lam2]);
    v0   = hypermat * V00;

    % Run the Gibbs sampler
    tau   = sqrt(0.15*var(yvar));
    bhat  = gibbs_lsq(niter,yvar,xvar,mu0,v0,sig0,tau);
    count = fwrite(fid,bhat,'single');
end
fclose(fid);

%% Core PCE inflation model

fname = strcat(loaddate,'auxcpce.dat'); delete(fname);
fid   = fopen(fname,'a');

% Sample data
cpce  = AY(:,2);

yvar   = cpce;
pistar = exp(mpara(19)/400);
 
% Prior is from regression of pi state on fundamental states

mu0 = inv(xvar'*xvar)*xvar'*smat(:,9);

% constant as suggested from measurement equation and parameter estimates
mu0 = [100*log(pistar); mu0];

% Prior variance from AR(1) regression
yar = yvar(2:end);
[nobs,nvar]=size(yar);
xar = yvar(1:end-1);
bar = inv(xar'*xar)*xar'*yar;
ear = yar - xar*bar;
sig0 = ear'*ear/nobs;

V00 = inv(diag([1;lambda']));

for ilam = 1:3
    
    disp(sprintf('\n'));
    disp(sprintf('%30s\t%2.1e','--core pce inflation-- lambda =',nlam(ilam)));
  
    % Set hyperparameters
    lam1 = nlam(ilam);
    lam2 = nlam(ilam);
    lam3 = nlam(ilam);
    hypermat = diag([lam1,lam2,lam3,lam2,lam2,lam2,lam2,lam2,lam2,lam2,lam2,lam2]);
    v0   = hypermat * V00;

    % Run the Gibbs sampler
    tau   = sqrt(0.15*var(yvar));
    bhat  = gibbs_lsq(niter,yvar,xvar,mu0,v0,sig0,tau);
    count = fwrite(fid,bhat,'single');
end
fclose(fid);

%% Unemployment rate

fname = strcat(loaddate,'auxur.dat'); delete(fname);
fid   = fopen(fname,'a');

% Sample data
ur  = AY(:,3);
yvar = ur;

Lstar = mpara(25);
%
%  Prior from regression of pi state on fundamental states

mu0 = inv(xvar'*xvar)*xvar'*(-smat(:,6));

% constant as suggested from measurement equation and parameter estimates
mu0 = [log(Lstar); mu0/3];

% Prior variance from AR(1) regression
yar = yvar(2:end);
[nobs,nvar]=size(yar);
xar = yvar(1:end-1);
bar = inv(xar'*xar)*xar'*yar;
ear = yar - xar*bar;
sig0 = ear'*ear/nobs;

V00 = inv(diag([1;lambda']));
sbhat = zeros(900,14);

for ilam = 1:3
    
    disp(sprintf('\n'));
    disp(sprintf('%30s\t%2.1e','--unemployment rate-- lambda =',nlam(ilam)));
  
    % Set hyperparameters
    lam1 = nlam(ilam);
    lam2 = nlam(ilam);
    lam3 = nlam(ilam);
    hypermat = diag([lam1,lam2,lam3,lam2,lam2,lam2,lam2,lam2,lam2,lam2,lam2,lam2]);

    v0   = hypermat * V00;

    % 
    % Run the Gibbs sampler
    tau   = sqrt(0.15*var(yvar));
    bhat  = gibbs_lsq(niter,yvar,xvar,mu0,v0,sig0,tau);
    count = fwrite(fid,bhat,'single');
end
fclose(fid);
