clear;

% Read in the data. The program auxest.m must have been run first to
% generate the simulation values

datadir = '../output/output-03-Dec-2008/';
nsim    = 100000;
nvar    = 14;
nobs    = nsim * nvar;
nskip   = 100;

% Sub sample the draws

% PCE inflation
fname         = strcat(datadir,'auxpce.dat');
[spc1,spc2,spc3] = SubSampleAuxDraws(fname,nsim,nskip,nvar);

% Core PCE Inflation
fname            = strcat(datadir,'auxcpce.dat');
[scpc1,scpc2,scpc3] = SubSampleAuxDraws(fname,nsim,nskip,nvar);

% Unemployment Rate
fname = strcat(datadir,'auxur.dat');
[sur1,sur2,sur3] = SubSampleAuxDraws(fname,nsim,nskip,nvar);

% Set up data. Forecasts are one-period ahead from end of sample
[M,YY,AY] = FcstSetup();
pc  = AY(:,1);
cpc = AY(:,2);
ur  = AY(:,3);

% Get subsample draws from DSGE model estimation
dname   = '../output/output-03-Dec-2008/smpl-1984Q1-2008Q3-block-';
nblocks = 100;
nsimb   = 10000;
nskip   = 100;
ndraws  = (nsimb*nblocks)/nskip;
npara   = 40;
pmat    = SubsampleDsgeDraws(dname,nblocks,nsimb,nskip,npara);

pmat    = pmat(0.2*ndraws:end,:);
mpara   = mean(pmat);

% Get DSGE model state variables
[smat,xvar,ypred,evec,lambda,retcode] = genrstates(mpara,YY,M);
state = smat(end,:);

mspc  = mean(spc2);
mscpc = mean(scpc2);
msur  = mean(sur2);

[niter,nvar] = size(spc1);
steps   = 24;

fPcShock  = zeros(niter,steps);
fCpcShock = zeros(niter,steps);
fUrShock  = zeros(niter,steps);

fPcFull  = zeros(niter,steps);
fCpcFull = zeros(niter,steps);
fUrFull  = zeros(niter,steps);

tic;
% Simulations for shock uncertainty
for i = 1:niter
    
    shocks  = randn(steps,1);
    epsDSGE = randn(steps,7);
    
    fpc = pdrawsAUX(mpara,mspc,pc(end,1),state,M,epsDSGE,shocks,steps);
    fPcShock(i,:) = fpc;
    
    fcpc = pdrawsAUX(mpara,mscpc,cpc(end,1),state,M,epsDSGE,shocks,steps);
    fCpcShock(i,:) = fcpc;
    
    fur = pdrawsAUX(mpara,msur,ur(end,1),state,M,epsDSGE,shocks,steps);
    fUrShock(i,:) = fur;
        
end

% Simulations for shock and parameter uncertainty
for i = 1:niter
    
    shocks  = randn(steps,1);
    epsDSGE = randn(steps,7);
    
    fpc2 = pdrawsAUX(mpara,spc1(i,:),pc(end,1),state,M,epsDSGE,shocks,steps);
    fPcFull(i,:) = fpc2;
    
    fcpc2 = pdrawsAUX(mpara,scpc1(i,:),cpc(end,1),state,M,epsDSGE,shocks,steps);
    fCpcFull(i,:) = fcpc2;
    
    fur2 = pdrawsAUX(mpara,sur1(i,:),ur(end,1),state,M,epsDSGE,shocks,steps);
    fUrFull(i,:) = fur2;
        
end

startDate = 2009.0;
ti = startDate : 0.25 : startDate + steps/4 - .25 ;

% ++++++++++++++++++ PCE Inflation results +++++++++++++++++++++++++++
% shock uncertainty
mvar1   = mean(fPcShock);
bounds = hpdint(fPcShock,0.68);
pcmat1 = 4 * [ bounds(1,:)', mvar1', bounds(2,:)' ];

% full uncertainty
mvar2   = mean(fPcFull);
bounds = hpdint(fPcFull,0.68);
pcmat2 = 4 * [ bounds(1,:)', mvar2', bounds(2,:)' ];

% ++++++++++++++++++ Core PCE Inflation results +++++++++++++++++++++++++++
% shock uncertainty
mvar3   = mean(fCpcShock);
bounds = hpdint(fCpcShock,0.68);
cpcmat1 = 4 * [ bounds(1,:)', mvar3', bounds(2,:)' ];

% full uncertainty
mvar4   = mean(fCpcFull);
bounds = hpdint(fCpcFull,0.68);
cpcmat2 = 4 * [ bounds(1,:)', mvar4', bounds(2,:)' ];

% ++++++++++++++++++ Unemployment Rate results +++++++++++++++++++++++++++
% shock uncertainty
mvar5  = mean(fUrShock);
bounds = hpdint(fUrShock,0.68);
urmat1 = [ bounds(1,:)', mvar5', bounds(2,:)' ];

% full uncertainty
mvar6   = mean(fUrFull);
bounds  = hpdint(fUrFull,0.68);
urmat2  = [ bounds(1,:)', mvar6', bounds(2,:)' ];

%++++++++++++++++++ Plots ++++++++++++++++++++++++++++++++++++++++++++++++
figure
subplot(2,2,1);
plot(ti',pcmat2);
title('PCE Inf w/68% coverage');

subplot(2,2,2);
plot(ti',4*mvar2');
title('PCE Inf Point Forecast');

subplot(2,2,3);
plot(ti',cpcmat2);
title('Core PCE Inf w/68% coverage');

subplot(2,2,4);
plot(ti',4*mvar4');
title('Core PCE Inf point forecast');
%+++++++++++++++++++++++++
figure;
subplot(2,2,1);
plot(ti',urmat2);
title('Unemployment Rate w/68% coverage');

subplot(2,2,2);
plot(ti',mvar6');
title('Unemployment Rate point forecast');