% This program generates the results in Table 2 of the paper. 
%
% Keith Sill
% November 2007
%
%

%path(path,'/home/keith/MATLAB');

% structure to hold para names, startvals, estim vals, and stds
paras = struct('name',...
    {'alp','zeta_p','iota_p','del','ups','Bigphi','s2','h','a2','nu_l',...
    'nu_m','zeta_w','iota_w','law','beta','psi1','psi2','rho_r','pistar',...
    'gam','Wadj','chi','laf','gstar','Ladj','rho_z','rho_phi','rho_chi',...
    'rho_laf','rho_mu','rho_b','rho_g','sig_z','sig_phi','sig_chi',...
    'sig_laf','sig_mu','sig_b','sig_g','sig_r'},...
    'sval',...
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},...
    'estval',...
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},...
    'stderr',...
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0});

% get the prior from csv file

[prior] = csvread('benchmark_prior.csv',16,1);
pshape   = prior(:,1);
pmean    = prior(:,2);
pstdd    = prior(:,3);
pmask    = prior(:,4);
pfix     = prior(:,5);
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

% get the maximization start vals from csv file and put them in the paras
% structure

[p0vals] = csvread('benchmark_start.csv',2,1);
for i = 1:length(p0vals)
    paras(i).sval = p0vals(i);
end

%% Load data parameter draws, calculate mean

% Form the reduced-state selection matrix
%   
% The state variable ordering is:
% y,c,i,k,,bar,L,m,mc,pi,R,rk,u,w,wtil,xi,xik,z,phi,mu,b,g,laf,Ec,Epi,
% Erk,Ew,Ewtil,Exi,Exik,Ei,yl,cl,il,wl,ml
%
% The 'true' state variables are c,i,kbar,R,w,z,phi,mu,b,g,laf which are
% indexed a%s 2,3,5,10,13,17,18,19,20,21,22
%
% note that pi is not a state when the estimated model shuts down inflation
% indexation, as we have done here.
%
M = zeros(35,11);
M(2,1)   = 1;
M(3,2)   = 1;
M(5,3)   = 1;
M(10,4)  = 1;
M(13,5)  = 1;
M(17,6)  = 1;
M(18,7)  = 1;
M(19,8)  = 1;
M(20,9)  = 1;
M(21,10) = 1;
M(22,11) = 1;

nestim  = 27;
nstep   = 12;
nblocks = 70;
nvar    = 7;
nsim    = 1000;
npara   = 40;
nskip   = 100;
    
ndraws   = floor(nblocks*nsim/nskip);
paramat  = zeros(nblocks*nsim,npara);
fcastmat = zeros(nestim+1,nvar,nstep);
ar1mat   = zeros(nestim+1,nvar,nstep);

trej = 0;

loaddate = 'output-17-Mar-2009/';
  
    
for j = 0:nestim
        
    trej = 0;
    
    disp(sprintf('%15s%d','iteration: ',j));

    % load estimation sample data that always starts at 1984Q1 and goes
    % through 2000Q4 + j quarters

    for i = 1:nblocks

        clear fname parasim;

        fname = strcat(loaddate,'smpl-1984Q1+',num2str(67+j),'--block-',num2str(i+(100-nblocks)));
        load(fname,'parasim','rej');
        ridx1 = (i-1)*nsim + 1;
        ridx2 = i*nsim;
        paramat(ridx1:ridx2,:) = parasim;
        trej = sum(rej) + trej;

    end

    % load actual data, last observation corresponds to 2000Q4+j
    str1 = 186+j;
    [YY,txt,raw] = xlsread('usModelData.xls','Sheet3',strcat('a119:h',num2str(str1)));
    
    v = zeros(nvar,nstep);
    
    for i = 1:ndraws
        
        idx = nskip * i;
        
        mpara = paramat(idx,:);

        % States and j-step ahead forecasts for each period of YY
        % smat is 1-step, ... 12-step ahead standing at 2000Q4+j
        [smat,DD,ZZ,retcode] = state_fcst2(mpara,YY,nstep);
        
        % ypred is 1-step, ..., 12-step ahead standing at 2000Q4+j
        ypred   = fcast_mvars2(DD,ZZ,smat);
        
        for k = 1:nvar
            v(k,:) = v(k,:) + ypred(k,:);
        end
        
    end
    
    % mean of draws for each variable and each step
    v = v / ndraws;
    
    % fcastmat contains 1-step, ..., 12-step ahead forecasts for all
    % variables. The initial forecast is for 2001Q1, so the first index of
    % fcastmat is the date at which forecasts are made, and so begin at
    % 2000Q4+j
    
    fcastmat(j+1,:,:) = v;
    
   % simple ar1 forecasts for each variable
   for var = 1:7
      yar1 = ar1_fcst2(YY(:,var),nstep);
      ar1mat(j+1,var,:) = yar1;
   end
end

%% actual data for forecast sample is 2001Q1 to 2007Q4

sa1  = xlsread('usModelData.xls','Sheet3','a187:h214');
sa2  = xlsread('usModelData.xls','Sheet3','a188:h214');
sa4  = xlsread('usModelData.xls','Sheet3','a190:h214');
sa12 = xlsread('usModelData.xls','Sheet3','a198:h214');
% 
dsg_rmse = [];

%% RMSE for DSGE model forecasts
for var = 1:7
    
    s1  = fcastmat(1:end,var,1)     - sa1(:,var);
    s2  = fcastmat(1:end-1,var,2)   - sa2(:,var);
    s4  = fcastmat(1:end-3,var,4)   - sa4(:,var);
    s12 = fcastmat(1:end-11,var,12) - sa12(:,var);
    
    rmse1  = sqrt( s1'*s1/max(size(s1)) );
    rmse2  = sqrt(s2'*s2/max(size(s2)));
    rmse4  = sqrt(s4'*s4/max(size(s4)));
    rmse12 = sqrt(s12'*s12/max(size(s12)));
    
    dsg_rmse(var,:) = [rmse1,rmse2,rmse4,rmse12];
end

%% RMSE from ar1 forecasts
% Note that these forecasts are simple forward iterations on a classically
% estimated AR(1). We aren't doing anything Bayesian here....

ar1_rmse = [];

for var = 1:7
    
    as1  = ar1mat(1:end,var,1)     - sa1(:,var);
    as2  = ar1mat(1:end-1,var,2)   - sa2(:,var);
    as4  = ar1mat(1:end-3,var,4)   - sa4(:,var);
    as12 = ar1mat(1:end-11,var,12) - sa12(:,var);

    armse1  = sqrt( as1'*as1/max(size(as1)) );
    armse2  = sqrt(as2'*as2/max(size(as2)));
    armse4  = sqrt(as4'*as4/max(size(as4)));
    armse12 = sqrt(as12'*as12/max(size(as12)));

    ar1_rmse(var,:) = [armse1,armse2,armse4,armse12];
    
end

fname =  'table2b';
save(fname,'fcastmat','ar1mat','dsg_rmse','ar1_rmse','sa1','sa2','sa4','sa12');

%%

load 'table2b';
dm = zeros(7,4,4);

dmLossFnIndex = 1;
dmTestType    = 'two-sided' ;

for var = 1:7
    
   dmForecast1 = fcastmat(1:end,var,1);
   dmForecast2 = ar1mat(1:end,var,1);
   dmActual    = sa1(:,var);

   [DM, MDM, DM_pval, MDM_pval] = ...
               Diebold_Mariano(dmForecast1, dmForecast2, dmActual, ...
                               dmLossFnIndex, dmTestType, 1);
   dm(var,1,:) = [DM,MDM,DM_pval,MDM_pval];                        
   
                           
   dmForecast1 = fcastmat(1:end-1,var,2);
   dmForecast2 = ar1mat(1:end-1,var,2);
   dmActual    = sa2(:,var);

   [DM, MDM, DM_pval, MDM_pval] = ...
               Diebold_Mariano(dmForecast1, dmForecast2, dmActual, ...
                               dmLossFnIndex, dmTestType, 2);
   dm(var,2,:) = [DM,MDM,DM_pval,MDM_pval];                        
   
   dmForecast1 = fcastmat(1:end-3,var,4);
   dmForecast2 = ar1mat(1:end-3,var,4);
   dmActual    = sa4(:,var);

   [DM, MDM, DM_pval, MDM_pval] = ...
               Diebold_Mariano(dmForecast1, dmForecast2, dmActual, ...
                               dmLossFnIndex, dmTestType, 4);
   dm(var,3,:) = [DM,MDM,DM_pval,MDM_pval];                        
   
   dmForecast1 = fcastmat(1:end-11,var,12);
   dmForecast2 = ar1mat(1:end-11,var,12);
   dmActual    = sa12(:,var);

   [DM, MDM, DM_pval, MDM_pval] = ...
               Diebold_Mariano(dmForecast1, dmForecast2, dmActual, ...
                               dmLossFnIndex, dmTestType, 12);
                
   dm(var,4,:) = [DM,MDM,DM_pval,MDM_pval];                  
%                            
end

dsgeMat = [dm(1,1,4),dm(1,2,4),dm(1,3,4),dm(1,4,4);
          dm(2,1,4),dm(2,2,4),dm(2,3,4),dm(2,4,4);
          dm(3,1,4),dm(3,2,4),dm(3,3,4),dm(3,4,4);
          dm(4,1,4),dm(4,2,4),dm(4,3,4),dm(4,4,4);
          dm(5,1,4),dm(5,2,4),dm(5,3,4),dm(5,4,4);
          dm(6,1,4),dm(6,2,4),dm(6,3,4),dm(6,4,4);
          dm(7,1,4),dm(7,2,4),dm(7,3,4),dm(7,4,4)];
      
fname =  'table2_dsge';
save(fname,'dsg_rmse','ar1_rmse','dsgeMat');
