% Generate autocorrelation from model simulated data and compare them to
% the data correlations
%
% Keith Sill
% February 2009
%

clear;

% Model setup file is run first
[M,YY,AY] = FcstSetup();

[nr,nc] = size(YY);

% Get subsample draws from DSGE model estimation
dname   = 'output-17-Mar-2009/smpl-1984Q1+95--block-';
nblocks = 100;
nsim    = 1000;
nskip   = 20;
ndraws  = (nsim*nblocks)/nskip;
npara   = 40;
pmat    = SubsampleDsgeDraws(dname,nblocks,nsim,nskip,npara);

pmat     = pmat(0.2*ndraws:end,:);

% now the auxiliary variable block
% Sub sample auxiliary variable parameter draws from the Gibbs sampler
% output

datadir = 'output-17-Mar-2009\';

% PCE inflation
nskip = 20;
fname  = strcat(datadir,'bhat-pce-3-27.mat');
spc2 = SubSampleAuxDraws(fname,nskip);
[mean(spc2),var(spc2)];

% Core PCE Inflation
fname  = strcat(datadir,'bhat-cpce-3-27.mat');
scpc2 = SubSampleAuxDraws(fname,nskip);
[mean(spc2),var(spc2)];

% Unemployment Rate\
fname  = strcat(datadir,'bhat-ur-3-27.mat');
sur2 = SubSampleAuxDraws(fname,nskip);
[mean(spc2),var(spc2)];

% Housing Starts
fname  = strcat(datadir,'bhat-hs-3-27.mat');
shs2 = SubSampleAuxDraws(fname,nskip);
[mean(spc2),var(spc2)];

Yvar0 = AY(end,:);

% nsim is the number of simulations, each for steps number of periods
nsim = 1000;
steps = 500;

% Simulate
tic;


% Simulate the DSGE model and Auxiliary equation models forward and then
% run a VAR on the simulation data. 

rho0mat = [];

for i = 1 : max(size(spc2))  %same number as data observations

    paraDSGE = pmat(i,:);
    paraAUX  = [spc2(i,:);scpc2(i,:);sur2(i,:);shs2(i,:)];
    
    % Generate state observations from Kalman filter
    [smat,xvar,ypred,evec,lambda,retcode] = genrstates(paraDSGE,YY,M);
    state = smat(end,:);
	
    epsDSGE = randn(steps,7);
    
    % simulations for shock uncertainty
	fmat1 = pdrawsFull(paraDSGE,paraAUX,Yvar0,state,M,epsDSGE,steps);
   
    xmat    = fmat1(:,400:500)';
    
    % compute and store sample correlations. Rows of rho matrix are indexed
    % by columns of first arg while cols of rho matrix are indexed by cols
    % of second arg.  
    rho0 = corr(xmat,xmat);
    rho1 = corr(xmat(2:end,:),xmat(1:end-1,:));
    rho2 = corr(xmat(3:end,:),xmat(1:end-2,:));
    rho3 = corr(xmat(4:end,:),xmat(1:end-3,:));
    rho4 = corr(xmat(5:end,:),xmat(1:end-4,:));
    
    rho0mat = [rho0mat;
            rho0(8,1),rho0(8,6),rho0(8,7), ...
            rho0(9,1),rho0(9,6),rho0(9,7), ... 
            rho0(10,1),rho0(10,6),rho0(10,7), ...
            rho0(11,1),rho0(11,6),rho0(11,7)];
    
    % calculate average recursively
    if i == 1
        mrho0 = rho0;
        mrho1 = rho1;
        mrho2 = rho2;
        mrho3 = rho3;
        mrho4 = rho4;
    else
        mrho0 = (i-1)/i * mrho0 + 1/i * rho0;
        mrho1 = (i-1)/i * mrho1 + 1/i * rho1;
        mrho2 = (i-1)/i * mrho2 + 1/i * rho2;
        mrho3 = (i-1)/i * mrho3 + 1/i * rho3;
        mrho4 = (i-1)/i * mrho4 + 1/i * rho4;
    end
        
    
    
    %results = vare(xmat,4);
    
    % store the VAR results for each simulation run
%     eq1mat  = [eq1mat;results(1).beta'];
%     eq2mat  = [eq2mat;results(2).beta'];
%     eq3mat  = [eq3mat;results(3).beta'];
%     eq4mat  = [eq4mat;results(4).beta'];
%     eq5mat  = [eq5mat;results(5).beta'];
%     eq6mat  = [eq6mat;results(6).beta'];
%     eq7mat  = [eq7mat;results(7).beta'];
%     eq8mat  = [eq8mat;results(8).beta'];
%     eq9mat  = [eq9mat;results(9).beta'];
%     eq10mat = [eq1mat;results(10).beta'];
    
end

dxmat = [YY,AY];
% compute and store sample correlations
drho0 = corr(dxmat,dxmat);
drho1 = corr(dxmat(2:end,:),dxmat(1:end-1,:));
drho2 = corr(dxmat(3:end,:),dxmat(1:end-2,:));
drho3 = corr(dxmat(4:end,:),dxmat(1:end-3,:));
drho4 = corr(dxmat(5:end,:),dxmat(1:end-4,:));

%% Print out a table of cross correlations 
type = 'Data';  % Data or Model
mmat  = mrho0;   % drhox for data or mrhox for Model
dmat  = drho0;
idx  = 't+2';   % leads string

disp(sprintf('\nModel Cross Correlations. Data in parenthesis\n'))
disp(sprintf('            y(t)      c(t)    I(t)     h(t)     w(t)     pgdp     R(t)     pce(t)   cpce(t)  u(t)'))
disp(sprintf('_________________________________________________________________________________________________'))
disp(sprintf('y(%3s)    | %5.2f     %5.2f    %5.2f    %5.2f    %5.2f    %5.2f    %5.2f    %5.2f    %5.2f    %5.2f',idx,mmat(1,1:10)))
disp(sprintf('          | (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)\n',dmat(1,1:10)))
disp(sprintf('c(%3s)    | %5.2f     %5.2f    %5.2f   %5.2f     %5.2f   %5.2f     %5.2f    %5.2f    %5.2f    %5.2f',idx,mmat(2,1:10)))
disp(sprintf('          | (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)\n',dmat(2,1:10)))
disp(sprintf('I(%3s)    | %5.2f     %5.2f    %5.2f   %5.2f     %5.2f   %5.2f     %5.2f    %5.2f    %5.2f    %5.2f',idx,mmat(3,1:10)))
disp(sprintf('          | (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)\n',dmat(3,1:10)))
disp(sprintf('h(%3s)    | %5.2f     %5.2f    %5.2f   %5.2f     %5.2f   %5.2f     %5.2f    %5.2f    %5.2f    %5.2f',idx,mmat(4,1:10)))
disp(sprintf('          | (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)\n',dmat(4,1:10)))
disp(sprintf('w(%3s)    | %5.2f     %5.2f    %5.2f   %5.2f     %5.2f   %5.2f     %5.2f    %5.2f    %5.2f    %5.2f',idx,mmat(5,1:10)))
disp(sprintf('          | (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)\n',dmat(5,1:10)))
disp(sprintf('pgdp(%3s) | %5.2f     %5.2f    %5.2f   %5.2f     %5.2f   %5.2f     %5.2f    %5.2f    %5.2f    %5.2f',idx,mmat(6,1:10)))
disp(sprintf('          | (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)\n',dmat(6,1:10)))
disp(sprintf('R(%3s)    | %5.2f     %5.2f    %5.2f   %5.2f     %5.2f   %5.2f     %5.2f    %5.2f    %5.2f    %5.2f',idx,mmat(7,1:10)))
disp(sprintf('          | (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)\n',dmat(7,1:10)))
disp(sprintf('pce(%3s)  | %5.2f     %5.2f    %5.2f   %5.2f     %5.2f   %5.2f     %5.2f    %5.2f    %5.2f    %5.2f',idx,mmat(8,1:10)))
disp(sprintf('          | (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)\n',dmat(8,1:10)))
disp(sprintf('cpce(%3s) | %5.2f     %5.2f    %5.2f   %5.2f     %5.2f   %5.2f     %5.2f    %5.2f    %5.2f    %5.2f',idx,mmat(9,1:10)))
disp(sprintf('          | (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)\n',dmat(9,1:10)))
disp(sprintf('u(%3s)    | %5.2f     %5.2f    %5.2f   %5.2f     %5.2f   %5.2f     %5.2f    %5.2f    %5.2f    %5.2f',idx,mmat(10,1:10)))
disp(sprintf('          | (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)  (%5.2f)',dmat(10,1:10)))


%% Print out a table of contemporary cross correlations 
% type = 'Data';  % Data or Model
% mmat  = mrho0;   % drhox for data or mrhox for Model
% dmat  = drho0;
% 
% disp(sprintf('\nModel Cross Correlations. Data in parenthesis\n'))
% disp(sprintf('        &      y     &    pgdp  &    R    '))
% disp(sprintf('_________________________________________________________________________________________________'))
% disp(sprintf('pce    &    %5.2f   &   %5.2f  &  %5.2f   '  ,mmat(8,1),mmat(8,6),mmat(8,7)))
% disp(sprintf('       &    (%5.2f) &  (%5.2f) & (%5.2f)  \n',dmat(8,1),dmat(8,6),dmat(8,7)))
% disp(sprintf('cpce   &    %5.2f   &   %5.2f  &  %5.2f   '  ,mmat(9,1),mmat(9,6),mmat(9,7)))
% disp(sprintf('       &    (%5.2f) &  (%5.2f) & (%5.2f)  \n',dmat(9,1),dmat(9,6),dmat(9,7)))
% disp(sprintf('u      &    %5.2f   &   %5.2f  &  %5.2f   '  ,mmat(10,1),mmat(10,6),mmat(10,7)))
% disp(sprintf('       &    (%5.2f) &  (%5.2f) & (%5.2f)  '  ,dmat(10,1),dmat(10,6),dmat(10,7)))
% disp(sprintf('hs     &    %5.2f   &   %5.2f  &  %5.2f   '  ,mmat(11,1),mmat(11,6),mmat(11,7)))
% disp(sprintf('       &    (%5.2f) &  (%5.2f) & (%5.2f)  '  ,dmat(11,1),dmat(11,6),dmat(11,7)))
% 
%%
rho0mean = mean(rho0mat);
hprho    = hpdint(rho0mat,0.90);

type = 'Data';  % Data or Model
% drhox for data or mrhox for Model
mrho = mean(rho0mat);
dmat  = drho0;

fname = strcat(date,'cross_corr');
save(fname,'mrho','hprho','dmat');


disp(sprintf('\nModel Cross Correlations. Data in parenthesis\n'))
disp(sprintf('        &      y     &    pgdp  &    R    '))
disp(sprintf('_________________________________________________________________________________________________'))
disp(sprintf('pce    &    %5.2f       &   %5.2f       &  %5.2f   '  ,mrho(1),mrho(2),mrho(3)))
disp(sprintf('pint   &  [%5.2f,%5.2f] & [%5.2f,%5.2f] & [%5.2f,%5.2f]',hprho(1,1),hprho(2,1),hprho(1,2),hprho(2,2),hprho(1,3),hprho(2,3)))
disp(sprintf('data   &    (%5.2f)     &  (%5.2f)      & (%5.2f)  \n',dmat(8,1),dmat(8,6),dmat(8,7)))
disp(sprintf('_________________________________________________________________________________________________'))
disp(sprintf('cpce   &    %5.2f       &   %5.2f       &  %5.2f   '  ,mrho(4),mrho(5),mrho(6)));
disp(sprintf('pint   &  [%5.2f,%5.2f] & [%5.2f,%5.2f] & [%5.2f,%5.2f]',hprho(1,4),hprho(2,4),hprho(1,5),hprho(2,5),hprho(1,6),hprho(2,6)))
disp(sprintf('data   &    (%5.2f)     &  (%5.2f)      & (%5.2f)  \n',dmat(9,1),dmat(9,6),dmat(9,7)))
disp(sprintf('_________________________________________________________________________________________________'))
disp(sprintf('u      &    %5.2f       &   %5.2f       &  %5.2f   '  ,mrho(7),mrho(8),mrho(9)));
disp(sprintf('pint   &  [%5.2f,%5.2f] & [%5.2f,%5.2f] & [%5.2f,%5.2f]',hprho(1,7),hprho(2,7),hprho(1,8),hprho(2,8),hprho(1,9),hprho(2,9)))
disp(sprintf('data   &    (%5.2f)     &  (%5.2f)      & (%5.2f)  '  ,dmat(10,1),dmat(10,6),dmat(10,7)))
disp(sprintf('_________________________________________________________________________________________________'))
disp(sprintf('hs     &    %5.2f       &   %5.2f       &  %5.2f   '  ,mrho(10),mrho(11),mrho(12)))
disp(sprintf('pint   &  [%5.2f,%5.2f] & [%5.2f,%5.2f] & [%5.2f,%5.2f]',hprho(1,10),hprho(2,10),hprho(1,11),hprho(2,11),hprho(1,12),hprho(2,12)))
disp(sprintf('data   &    (%5.2f)     &  (%5.2f)      & (%5.2f)  '  ,dmat(11,1),dmat(11,6),dmat(11,7)))
disp(sprintf('_________________________________________________________________________________________________'))

