function z = fcast_auxvars2(para,Yvar0,smat,M,shocks,step)
%
% Forecast auxiliary variables.
%
% The program is set up on the assumption that the first row of the array
% smat contains the last actual observations (ie, it is assumed already
% realized. The model used to make the forecasts is of the form:
%
%          z(t) = rho*z(t-1) + (1-rho)*a0 + (s(t)-s(t-1)*rho)*a1 + eta
% where
%          s(t) = M * states(t)
%
%

[nt,sv,st] = size(smat);

a0  = para(1);
a1  = para(2:12);
rho = para(13);
sg  = sqrt(para(14));

z = zeros(step+1,1);
z(1) = Yvar0;

for i = 1:step
    
    z(i+1) = rho * z(i) + a0*(1-rho) + ...
        (smat(end,:,i+1) - rho .*smat(end,:,i)) * M * a1' +  sg*shocks(i,1);
    
end

z = z(2:end);




