% Model Diagnostics
%
% Keith Sill
% December 2007

% Load model data used to generate state variables

%clear;
%path(path,'/home/keith/MATLAB');

clear;

mean_mat = zeros(70,40);

% load parameter values from posterior simulator and calculate mean
% nblocks is the number of blocks to be read out of 100. Thus, 100-nblocks
% is the "burn-in"

% nblocks = 70;
% nsim    = 10000;
% npara   = 40;
% 
% paramat = zeros(nblocks*nsim,npara);
% 
% for i = 1:nblocks
%     
%     clear fname parasim;
%     
%     fname = strcat('output/30-Apr-2008-1984Q1-2005Q4-block-',num2str(i+(100-nblocks)));
%     load(fname,'parasim');
%     ridx1 = (i-1)*nsim + 1;
%     ridx2 = i*nsim;
%     paramat(ridx1:ridx2,:) = parasim;
%     
% end



nblocks = 70;
nsim    = 1000;
npara   = 40;

paramat = zeros(nblocks*nsim,npara);
trej = 0;

loaddate = 'output-17-Mar-2009/';

for i = 1:nblocks

    clear fname parasim;

    fname = strcat(loaddate,'smpl-1984Q1+',num2str(94),'--block-',num2str(i+(100-nblocks)));
    load(fname,'parasim','rej');
    ridx1 = (i-1)*nsim + 1;
    ridx2 = i*nsim;
    paramat(ridx1:ridx2,:) = parasim;
    trej = sum(rej) + trej;

end


%% Parameter Estimates and Standard Errors

% mean and standard deviation of draws
mpara = mean(paramat);
spara = std(paramat);
rej_pct = trej/(nblocks*nsim);

% structure to hold para names, startvals, estim vals, and stds
paras = struct('name',...
    {'alp','zeta_p','iota_p','del','ups','Bigphi','s2','h','a2','nu_l',...
    'nu_m','zeta_w','iota_w','law','beta','psi1','psi2','rho_r','pistar',...
    'gam','Wadj','chi','laf','gstar','Ladj','rho_z','rho_phi','rho_chi',...
    'rho_laf','rho_mu','rho_b','rho_g','sig_z','sig_phi','sig_chi',...
    'sig_laf','sig_mu','sig_b','sig_g','sig_r'},...
    'sval',...
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},...
    'estval',...
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},...
    'stderr',...
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0});

% get the prior from csv file

[prior] = csvread('benchmark_prior.csv',16,1);
pshape   = prior(:,1);
pmean    = prior(:,2);
pstdd    = prior(:,3);
pmask    = prior(:,4);
pfix     = prior(:,5);
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

% get the maximization start vals from csv file and put them in the paras
% structure

[p0vals] = csvread('benchmark_start.csv',2,1);
for i = 1:length(p0vals)
    paras(i).sval = p0vals(i);
end

% fill the structure with mean of draws and std of draws
for i = 1:npara
    paras(i).estval = mpara(i);
    paras(i).stderr = spara(i);
end

% structure to hold values governing para fixity
varargin = struct('pmaskinv',pmaskinv,'pfix',pfix,'pmask',pmask,'pmean',...
     pmean,'pstdd',pstdd,'pshape',pshape);
 
% % display the estimates, standard errors, start values, prior mean, prior
% % std, whether para is fixed, and value fixed at (if fixed)
% disp(sprintf('%15s%15s%15s%15s%15s%15s%15s%15s','Parameter','Estimate','Std','Start-V','Prior Mean',...
%     'Prior Std','mask','fixed'));
% for i = 1:npara
%     disp(sprintf('%15s%15.5g%15.5g%15.5g%15.5g%15.5g%15.5g%15.5g',paras(i).name,paras(i).estval,paras(i).stderr,...
%         paras(i).sval,varargin.pmean(i),varargin.pstdd(i),varargin.pmask(i),varargin.pfix(i)));
% end
% 
% disp(sprintf('%15s%15g','Rejection Pct',rej_pct));

%% Model residuals
%
% Form the reduced-state selection matrix
%
% The state variable ordering is:
% y,c,i,k,,bar,L,m,mc,pi,R,rk,u,w,wtil,xi,xik,z,phi,mu,b,g,laf,Ec,Epi,
% Erk,Ew,Ewtil,Exi,Exik,Ei,yl,cl,il,wl,ml
%
% The 'true' state variables are c,i,kbar,R,w,z,phi,mu,b,g,laf which are
% indexed as 2,3,5,10,13,17,18,19,20,21,22
%
% note that pi is not a state when the estimated model shuts down inflation
% indexation, as we have done here.
%
M = zeros(35,11);
M(2,1)   = 1;
M(3,2)   = 1;
M(5,3)   = 1;
M(10,4)  = 1;
M(13,5)  = 1;
M(17,6)  = 1;
M(18,7)  = 1;
M(19,8)  = 1;
M(20,9)  = 1;
M(21,10) = 1;
M(22,11) = 1;

%M = eye(35);

% Generate states, fundamental states, predicted yvars, resids from state
% space model, unconditional variances of para estimatates, return code

% load estimation sample data 1984Q1 to 2007Q3
str1 = 186+27;
[YY,txt,raw] = xlsread('usModelData.xls','Sheet3',strcat('a119:h',num2str(str1)));

[smat,xmat,ypred,evec,lambda,retcode] = genrstates(mpara,YY,M);
[r,c] = size(evec);
% smat = smat(2:r,:);
% xmat = xmat(2:r,:);
% evec = evec(2:r,:);
ti = 1984.0:0.25:2007.5;

%% Figure1

[nr,nc] = size(xmat);
% Plot the fundamental states
figure('Name','State Variable Plots');

subplot(3,2,1);
plot(ti',xmat(:,6),ti',zeros(nr,1));
axis([min(ti),max(ti),-2,2]);
title('a');

subplot(3,2,2);
plot(ti',xmat(:,7),ti',zeros(nr,1));
axis([min(ti),max(ti),-40,20]);
title('{\phi}');

subplot(3,2,3);
plot(ti',xmat(:,8),ti',zeros(nr,1));
axis([min(ti),max(ti),-2,4]);
title('{\mu}');

subplot(3,2,4);
plot(ti',xmat(:,9),ti',zeros(nr,1));
axis([min(ti),max(ti),-2,4]);
title('b');

subplot(3,2,5);
plot(ti',xmat(:,10),ti',zeros(nr,1));
axis([min(ti),max(ti),-4,2]);
title('g');

subplot(3,2,6);
plot(ti',xmat(:,11),ti',zeros(nr,1));
axis([min(ti),max(ti),-0.5,1]);
title('{\lambda}_{f}');

%% Figure 2

% Load the auxiliary variables
% Estimation sample data 1984Q1 to 2000Q3 
str2 = 168+27;
[data1,txt1,raw1] = xlsread('auxvars2.xls','Sheet2',strcat('a101:e',num2str(str2)));

%idx2 = 101+67+27;
%data1 = csvread('auxvars.csv',101,1,[101,1,idx2,3]);

pgdp = YY(:,6);
hrs  = YY(:,4);
dsghs  = smat(:,3);

mhrs = mean(hrs);
hrs = -mhrs + 100/3 * (hrs-mhrs);

pce  = data1(:,1);
cpce = data1(:,2);
ur   = data1(:,3);
hs   = data1(:,4)/1000;

[r,c] = size(hs);
mhs   = mean(hs);
hs    = (hs - mhs);

figure('Name','Data Plots');
subplot(2,2,1);
plot(ti',pce, 'b--',ti',pgdp,'g-');
legend('pce','{\pi}','Location','SouthWest');
legend boxoff;
title('PCE Inflation, Inflation');

subplot(2,2,2);
plot(ti',cpce,'b--',ti',pgdp,'g-');
legend('cpce','{\pi}','Location','SouthWest');
legend boxoff;
title('Core PCE Inflation, Inflation');

subplot(2,2,3);
plot(ti',ur,'b--',ti',-hrs,'g-');
legend('ur','-0.31 L','Location','SouthWest');
legend boxoff;
title('Unemployment Rate, Hours');

subplot(2,2,4);
plot(ti',hs,'b--',ti',0.033*dsghs,'g-');
legend('hs','0.033 i','Location','SouthWest');
legend boxoff;
title('Housing Starts, Investment');