% Driver program for in sample fit of estimated DSGE model
%
% Keith Sill
% November 2007

% Load model data used to generate state variables
[YY,txt]        = loaddata();
txt
[nobs,ny] = size(YY);

mean_mat = zeros(70,40);

% load parameter values from posterior simulator and calculate mean
% nblocks is the number of blocks to be read out of 100. Thus, 100-nblocks
% is the "burn-in"

nblocks = 70;

for i = 1:nblocks
    
    clear fname parasim pvec;
    
    fname = strcat('output\03-Jan-2008-1984Q1-2005Q4-block-',num2str(i+(100-nblocks)));
    load(fname,'parasim');
    pvec = mean(parasim);
    mean_mat(i,:) = pvec;
    
end

para = mean(mean_mat);

% Form the reduced-state selection matrix
%
% The state variable ordering is:
% y,c,i,k,,bar,L,m,mc,pi,R,rk,u,w,wtil,xi,xik,z,phi,mu,b,g,laf,Ec,Epi,
% Erk,Ew,Ewtil,Exi,Exik,Ei,yl,cl,il,wl,ml
%
% The 'true' state variables are c,i,kbar,R,w,z,phi,mu,b,g,laf which are
% indexed as 2,3,5,10,13,17,18,19,20,21,22
%
% note that pi is not a state when the estimated model shuts down inflation
% indexation, as we have done here.
%
M = zeros(35,11);
M(2,1)   = 1;
M(3,2)   = 1;
M(5,3)   = 1;
M(10,4)  = 1;
M(13,5)  = 1;
M(17,6)  = 1;
M(18,7)  = 1;
M(19,8)  = 1;
M(20,9)  = 1;
M(21,10) = 1;
M(22,11) = 1;

[smat,xvar,ypred,evec,lambda,retcode] = genrstates(para,YY,M);
[nobs,nstates] = size(xvar);
% State space representation at estimated parameter vector
%       S(t) = TT*S(t-1) + RR*e(t)
%       e(t) ~ iid N(0,QQ)
%       y(t) = DD + ZZ*x(t) + u(t)
%       u(t) ~ N(0,HH)
%  
%       cov(e(t),u(t)) = VV
%
% The system shock vector e(t) is ordered as:
%   [ z   ;
%     phi ;
%     mu  ;
%     b   ;
%     g   ;
%     laf ;
%     R   ]
%
% The observation vector y(t) is:
%   [  real per-capita GDP growth quarterly percent;
%      real per-capita consumption growth quarterly percent;
%      real per-capita investment growth quarterly percent;
%      log hours worked per capita quarterly;
%      log hourly real wage;
%      GDP deflator inflation annualized percent;
%      nominal fed funds rate annualized percent  ]
ti = 1984.0:0.25:2005.75;

