% Predictive distribution script


clear;
%%

% Directory where DSGE model parameter estimates live
loaddate = 'output-17-Mar-2009/';
trej = 0;

nblocks = 70;
nsim    = 1000;
npara   = 40;

% Number of iterations for Gibbs sampler
niter   = 10000;
% Subsample frequency
nskip   = 50;
% Drop the first burnin before sampling
burnin  = 1000;

% Number of parameter draws from Gibbs
ndraws  = floor((niter-burnin)/nskip);

% max forecast horizon
nsteps  = 12;

% number of quarters ahead from 2000Q4 that estimation will roll
nestim = 27;

% number of hyperparamter values for which distributions will be generated
nlam = 3;

% results storage array. Dimensioned by number of hyperparameters, number
% of quarters rolled through, maximum forecast horizon, number of variables
% for which simulations will be conducted (here, 7 dsge measurement vars +
% 4 auxiliary vars)
x1 = zeros(nlam,nestim+1,nsteps,ndraws,11);

% Form the reduced-state selection matrix
%
% The state variable ordering is:
% y,c,i,k,,bar,L,m,mc,pi,R,rk,u,w,wtil,xi,xik,z,phi,mu,b,g,laf,Ec,Epi,
% Erk,Ew,Ewtil,Exi,Exik,Ei,yl,cl,il,wl,ml
%
% The 'true' state variables are c,i,kbar,R,w,z,phi,mu,b,g,laf which are
% indexed as 2,3,5,10,13,17,18,19,20,21,22
%
% note that pi is not a state when the estimated model shuts down inflation
% indexation, as we have done here.
%
M = zeros(35,11);
M(2,1)   = 1;
M(3,2)   = 1;
M(5,3)   = 1;
M(10,4)  = 1;
M(13,5)  = 1;
M(17,6)  = 1;
M(18,7)  = 1;
M(19,8)  = 1;
M(20,9)  = 1;
M(21,10) = 1;
M(22,11) = 1;

for ilam = 1:3

    for j = 0:nestim
        
        disp(sprintf('%15s%10d%15s%10d','estim index: ',j,'ilam index: ',ilam));

        % Read in draws from DSGE model estimation 1984Q1 to 2000Q4+j
        for i = 1:nblocks

                clear fname parasim;

                fname = strcat(loaddate,'smpl-1984Q1+',num2str(67+j),'--block-',num2str(i+(100-nblocks)));
                load(fname,'parasim','rej');
                ridx1 = (i-1)*nsim + 1;
                ridx2 = i*nsim;
                paramat(ridx1:ridx2,:) = parasim;
                trej = sum(rej) + trej;

        end

        % Read in draws from auxiliary variable regressions
        % 1984Q1 to 2000Q4+j. These parameter draws are generated from the
        % programs table5_xx2.m
        fname = strcat('output-17-Mar-2009\','bhat-ur-',num2str(ilam),'-',num2str(j));
        load(fname,'bhat');
        urmat = bhat(1:end,:);

        fname = strcat('output-17-Mar-2009\','bhat-pce-',num2str(ilam),'-',num2str(j));
        load(fname,'bhat');
        pcemat = bhat(1:end,:);

        fname = strcat('output-17-Mar-2009\','bhat-cpce-',num2str(ilam),'-',num2str(j));
        load(fname,'bhat');
        cpcemat = bhat(1:end,:);
        
        fname = strcat('output-17-Mar-2009\','bhat-hs-',num2str(ilam),'-',num2str(j));
        load(fname,'bhat');
        hsmat = bhat(1:end,:);


        % load estimation sample data 1984Q1 to 2000Q4+j
        str1 = 186+j;
        [YY,txt,raw] = xlsread('usModelData.xls','Sheet3',strcat('a119:h',num2str(str1)));

        str2 = 168+j;
        [data1,txt1,raw1] = xlsread('auxvars2.xls','Sheet2',strcat('a101:e',num2str(str2)));

        % last observation of the period, used to initialize
        % quasi-difference equation for auxiliary variable simulations
        pce0  = data1(end,1);
        cpce0 = data1(end,2);
        ur0   = data1(end,3);
        hs0   = data1(end,4);

        % Now sample from the parameter draws. First, initialize some
        % vectors to zero. Results will be stored in these, they will grow
        % row-wise with each iteration

        yvar    = zeros(ndraws,nsteps);
        cvar    = zeros(ndraws,nsteps);
        ivar    = zeros(ndraws,nsteps);
        hvar    = zeros(ndraws,nsteps);
        wvar    = zeros(ndraws,nsteps);
        pvar    = zeros(ndraws,nsteps);
        rvar    = zeros(ndraws,nsteps);
        urvar   = zeros(ndraws,nsteps);
        pcevar  = zeros(ndraws,nsteps);
        cpcevar = zeros(ndraws,nsteps);
        hsvar   = zeros(ndraws,nsteps);
        
        %+++++++++++++++ REMOVE ++++++++++++++++++
        %pDSGE = mean(paramat);
        %[smat,xvar,ypred,evec,lambda,retcode] = genrstates(pDSGE,YY,M);
        %state = smat(end,:);
        %+++++++++++++++++++++++++++++++++++++++++
%         pDSGE = mean(paramat(burnin:end,:));
%         pUR   = mean(urmat(burnin:end,:)); 
%         pCPCE = mean(cpcemat(burnin:end,:)); 
%         pPCE  = mean(pcemat(burnin:end,:)); 
%         pHS   = mean(hsmat(burnin:end,:)); 

            for i = 1:ndraws

            % Draw parameters from DSGE and Gibbs 
            
            pDSGE = paramat(burnin+i*nskip,:);
                        
            pUR   = urmat(burnin+i*nskip,:);
            pCPCE = cpcemat(burnin+i*nskip,:);
            pPCE  = pcemat(burnin+i*nskip,:);
            pHS   = hsmat(burnin+i*nskip,:);

            % use Kalman to generate latent state at sample end period given
            % a parameter draw
            [smat,xvar,ypred,evec,lambda,retcode] = genrstates(pDSGE,YY,M);
            state = smat(end,:);

            % Draw shocks for simulation
            epsDSGE = randn(nsteps,7);
            epsAUX  = randn(nsteps,4);

            % Generate simulated data on DSGE and Auxiliary variables for steps
            % ahead 1 through nsteps
            ymat     = pdrawsDSGE(pDSGE,state,epsDSGE,nsteps);
            zvecUR   = pdrawsAUX(pDSGE,pUR,ur0,state,M,epsDSGE,epsAUX(:,1),nsteps);
            zvecPCE  = pdrawsAUX(pDSGE,pPCE,pce0,state,M,epsDSGE,epsAUX(:,2),nsteps);
            zvecCPCE = pdrawsAUX(pDSGE,pCPCE,cpce0,state,M,epsDSGE,epsAUX(:,3),nsteps);
            zvecHS   = pdrawsAUX(pDSGE,pHS,hs0,state,M,epsDSGE,epsAUX(:,4),nsteps);
            
            % Stack the simulated data into a time series. Each column is a step
            % ahead running from 1 to nsteps
            yvar(i,:)    = ymat(1,:);
            cvar(i,:)    = ymat(2,:);
            ivar(i,:)    = ymat(3,:);
            hvar(i,:)    = ymat(4,:);
            wvar(i,:)    = ymat(5,:);
            pvar(i,:)    = ymat(6,:);
            rvar(i,:)    = ymat(7,:);
            urvar(i,:)   = zvecUR';
            pcevar(i,:)  = zvecPCE';
            cpcevar(i,:) = zvecCPCE';
            hsvar(i,:)   = zvecHS';
        end
        
        % collect the results in the x1 array --- note the ordering
        for st = 1:nsteps;

            x1(ilam,j+1,st,:,:) = [yvar(:,st),cvar(:,st),ivar(:,st),hvar(:,st),...
                        wvar(:,st),pvar(:,st),rvar(:,st),...
                        pcevar(:,st),cpcevar(:,st),urvar(:,st),hsvar(:,st)];
        end

    end
end

% save the results, prefixed by date of run
date = '18-May-2009';
fname = strcat('preDist-',date);
save(fname,'x1');
