% Gibbs sampler test case 

% Keith Sill
% October 2007

function omat = reduced_gibbs_lsq(niter,data,states,mu0,v0,sig0,tau)

    [nobs,ny] = size(data);
    [nvars,nstates] = size(states);
    
    omat = zeros(niter,nstates+3);
    
    alp = mu0 + chol(v0)'*randn(nstates+1,1);
    rho = rand(1);
    sig2 = sig0;
    
    v0i = inv(v0);
    
    for i = 1:niter
        
        % find the conditional posterior of alpha given rho and sigeta
        sig2s = sqrt(sig2);
        y = [data(2:nobs,:) - rho*data(1:nobs-1,:);(data(1,:)*sig2s/tau)];
        
        x = [ones(nobs-1,1).*(1-rho),(states(2:nobs,:)-rho.*states(1:nobs-1,:));
            sig2s/tau, sig2s/tau * states(1,:)];
        
        xtx = x'*x;
        xty = x'*y;
        ixtx = inv(xtx);
        alphat = ixtx*xty;
        
        vat = inv(v0i + 1/sig2 * xtx);
        muat = vat*(v0i*mu0 + 1/sig2 * xtx*alphat);
        alp = muat + chol(vat)'*randn(nstates+1,1);
        
        ssqr = (y - x*alphat)'*(y-x*alphat);
        
        if sqrt(ssqr) > eps
        
            % find the conditional posterior of rho given alpha and sigeta
        
            y = data(2:nobs,:) - ([ones(nobs-1,1),states(2:nobs,:)])*alp;
            x = data(1:nobs-1,:) - ([ones(nobs-1,1),states(1:nobs-1,:)]*alp);
            
            xtx = x'*x;
      
            xty = x'*y;
            ixtx = inv(xtx);
            mur = ixtx*xty;
           
            vrt = sig2*ixtx;
            %vrtsq = chol(vrt)';
                    
            %rho = randraw('normaltrunc', [-1, 1, mur, vrtsq], 1 );
            rho = normt_rnd(mur,vrt,-1,1);
            
           if rho == -Inf
             rho = -1 + eps;
           end

        else
            
            rho = 0.0;
            
        end
                
              
%         % find the conditional posterior of sigeta given alpha and rho
%         
%         eta = data(2:nobs,:) - (rho*data(1:nobs-1,:)+alp(1)*(1-rho)+...
%             (states(2:nobs,:)-rho*states(1:nobs-1,:))*alp(2:nstates+1));
%         
%         sesq = eta'*eta;
%         z = sqrt(1/sesq)*randn(nobs-1,1);
%         sig2 = 1/(z'*z);

        
        omat(i,:) = [alp',rho];
        
        
    end
        
 return
 
 %////////////////////////////////////////////////////////////////////
%//
%// Random draw from truncated normal distribution
%//
%////////////////////////////////////////////////////////////////////

function fx = normt_rnd(mu,sigma2,left,right)


    std = sqrt(sigma2);

    nobs = max(size(mu));
    
    %// calculate bounds on probabilities
    lowerProb = Phi((left-mu)./std);
    upperProb = Phi((right-mu)./std);

    %// Draw uniform from within (lowerProb,upperProb)
    u = lowerProb + (upperProb-lowerProb).*rand(nobs,1);

    %// Find quantiles
    fx = mu + Phiinv(u).*std;
return
 

%////////////////////////////////////////////////////////
%//
%// compute standard normal quantile function of the vector x (0,1)
%//
%////////////////////////////////////////////////////////

function fx = Phiinv(x)

    fx = sqrt(2)*erfinv(2*x-1);
return


%//////////////////////////////////////////////////////////
%//
%// standard normal distribution function at x
%//
%/////////////////////////////////////////////////////////

function fx = Phi(x)

    fx = 0.5*(1+erf(x/sqrt(2)));
return

 

    